//-------------------------------------------------------------------- 2014-02-12 ---------
// 圧力計シールド V1
//-------------------------------------------------------- written by iizuka --------------
#include <MsTimer2.h>
#include <EEPROM.h>

#define PRESSURE_GAUGE 18   // Port Analog 4

#define LED_SEG_A       0   // Port Digial 0
#define LED_SEG_B       1   // Port Digial 1
#define LED_SEG_C       2   // Port Digial 2
#define LED_SEG_D       3   // Port Digial 3
#define LED_SEG_E       4   // Port Digial 4
#define LED_SEG_F       5   // Port Digial 5
#define LED_SEG_G       6   // Port Digial 6
#define LED_SEG_DP      7   // Port Digial 7

#define LED_DIG_1      12   // Port Digital 12   7seg digit 10
#define LED_DIG_2      11   // Port Digital 11   7seg digit 1
#define LED_5MM        13   // Port Digital 13   5mmLED

#define ENCODER_A       9   // Port Digital 9
#define ENCODER_B       8   // Port Digital 8

#define SLIDE_SW1      10   // Port Digital 10
#define TACT_SW2       17   // Port Analog  3
#define TACT_SW3       16   // Port Analog  2

#define REF_VOLTAGE    19   // Port Analog 5  TL431

#define PORT_AC1       14   // Port Analog 0  AC100V
#define PORT_AC2       15   // Port Analog 1  AC100V

// 5mm LED
#define LED1   0b10000000  // bit7
#define LED2   0b01000000  // bit6
#define LED3   0b00100000  // bit5
#define LED4   0b00010000  // bit4
#define LED5   0b00001000  // bit3
#define LED6   0b00000100  // bit2
#define LED7   0b00000010  // bit1
#define LED8   0b00000001  // bit0

// #define LED_ON   1
// #define LED_OFF  0

#define SW_ON    LOW
#define SW_OFF   HIGH

#define DIMMER  4   // dimmer level  0:bright <--> 7:dark  ７以上にすると点滅していることがわかる

// LEDセグメント点灯パターン
const byte Digit_0     = 0x3F;
const byte Digit_1     = 0x06;
const byte Digit_2     = 0x5B;
const byte Digit_3     = 0x4F;
const byte Digit_4     = 0x66;
const byte Digit_5     = 0x6D;
const byte Digit_6     = 0x7D;
const byte Digit_7     = 0x07;
const byte Digit_8     = 0x7F;
const byte Digit_9     = 0x6F;
const byte Digit_A     = 0x77;
const byte Digit_b     = 0x7C;
const byte Digit_C     = 0x39;
const byte Digit_d     = 0x5E;
const byte Digit_E     = 0x79;
const byte Digit_F     = 0x71;
const byte Digit_L     = 0x38;
const byte Digit_NON   = 0x00;
const byte Digit_MINUS = 0x40;
const byte Digit_DP    = 0x80;

// LED点灯関連
byte led_dig1;   // LED点灯パターン格納変数　１０桁目
byte led_dig2;   // LED点灯パターン格納変数　１桁目
byte led_5mm;    // LED点灯パターン格納変数　５ｍｍＬＥＤ
byte led_pos;    // LED 桁表示位置
byte led_dimmer; // LED明るさ制御用カウンタ

// ロータリーエンコーダ関連
#define CW            1
#define CCW           2
#define STOP          3
#define ENCODER_STOP  0
#define ROTARY_LOCK   1
#define ROTARY_UNLOCK 0
byte    encoder_A;
byte    encoder_B;
boolean encoder_A_lock;
boolean encoder_B_lock;
byte    encoder_stat;

// 圧力センサー設定関連
int  pressure_1atm_adcon;   // 1気圧におけるセンサーのＡＤコンバーター計測値
int  pressure_low_adcon;    // 低気圧におけるセンサーのＡＤコンバーター計測値
byte pressure_low_gauge;    //　　そのときのゲージ圧（実際の圧力）
int  pressure_high_adcon;   // 高気圧におけるセンサーのＡＤコンバーター計測値
byte pressure_high_gauge;   //　　そのときのゲージ圧（実際の圧力）
byte pressure_ac_on;        // コンプレッサーをＯＮする圧力　圧力がこれ以下でＯＮ
byte pressure_ac_off;       // コンプレッサーをＯＦＦする圧力　圧力がこれ以上でＯＦＦ

byte pressure_mode; // 使用目的により圧力を３パターン選べる
#define MODE_PRESSURE_LOW    1
#define MODE_PRESSURE_MIDIUM 2
#define MODE_PRESSURE_HIGH   3

volatile unsigned int disp_updt_counter;

// 圧力センサー　ADP1181 用初期値
#define DEFAULT_1ATM         0   //　１気圧のときのセンサーＡＤコンバータ計測値     
#define DEFAULT_LOW_ADCON    74  //　３．０気圧の時のセンサーＡＤコンバータ計測値　ADP1291
#define DEFAULT_LOW_GAUGE    30  //　３．０気圧
#define DEFAULT_HIGH_ADCON   287 //　５．０気圧の時のセンサーＡＤコンバータ計測値　ADP1291
#define DEFAULT_HIGH_GAUGE   50  //　５．０気圧
#define DEFAULT_AC_ON_L      30  //　コンプレッサーをＯＮする気圧　　３．０気圧
#define DEFAULT_AC_OFF_L     60  //　コンプレッサーをＯＦＦする気圧　６．０気圧
#define DEFAULT_AC_ON_M      50  //　コンプレッサーをＯＮする気圧　　５．０気圧
#define DEFAULT_AC_OFF_M     70  //　コンプレッサーをＯＦＦする気圧　７．０気圧
#define DEFAULT_AC_ON_H      70  //　コンプレッサーをＯＮする気圧　　７．０気圧
#define DEFAULT_AC_OFF_H     75  //　コンプレッサーをＯＦＦする気圧　７．５気圧
#define DEFAULT_MODE         MODE_PRESSURE_LOW   //　モード初期値

// 圧力センサー用初期値のＥＥＰＲＯＭ格納アドレス
#define ADDRESS_1ATM_AD          0
#define ADDRESS_LOW_AD_LOWBYTE   1   
#define ADDRESS_LOW_AD_HIGHBYTE  2
#define ADDRESS_HIGH_AD_LOWBYTE  3   
#define ADDRESS_HIGH_AD_HIGHBYTE 4
#define ADDRESS_LOW_GAUGE        5
#define ADDRESS_HIGH_GAUGE       6
#define ADDRESS_AC_ON_L          7
#define ADDRESS_AC_OFF_L         8
#define ADDRESS_AC_ON_M          9
#define ADDRESS_AC_OFF_M        10
#define ADDRESS_AC_ON_H         11
#define ADDRESS_AC_OFF_H        12
#define ADDRESS_MODE            13

//------------------------------------------------------------------------------------------
// EEPROMが初期設定されているか否かを返す 初期化の有無はchecksumによる
//------------------------------------------------------------------------------------------
boolean eeprom_is_initialized()
{
  byte sum;
  byte check_sum;
  
  sum = 0;
  for( int address = 0 ; address < 0xFF ; address++ ) { 
    sum += EEPROM.read( address );
  }
  check_sum = EEPROM.read( 0xFF );
  if( sum == check_sum ) {
    return true;
  } else {
    return false;
  }
}

//------------------------------------------------------------------------------------------
// EEPROMが初期設定されているかチェックする
//------------------------------------------------------------------------------------------
void eeprom_write_checksum()
{
  byte sum;
  
  sum = 0;
  for( int address = 0 ; address < 0xFF ; address++ ) {
    sum += EEPROM.read( address );
  }

  EEPROM.write( 0xFF, sum );
}

//------------------------------------------------------------------------------------------
// ロータリーエンコーダ
// return    CW / CCW / OTHER
//------------------------------------------------------------------------------------------
byte rotary_encoder()
{
  byte temp_A;
  byte temp_B;
  byte result;

  temp_A = digitalRead( ENCODER_A );
  temp_B = digitalRead( ENCODER_B );
 
  result = ENCODER_STOP;

  if( (encoder_A == 1) && (temp_A == 0) && (temp_B == 1) ) {
    encoder_stat   = CW;
    encoder_A_lock = true;
  }
  if( encoder_A_lock ) {
    if( (temp_A == 0) && (temp_B == 0) ) {
      encoder_A_lock = false;
      result = encoder_stat;
    }
  }

  if( (encoder_B == 1) && (temp_B == 0) && (temp_A == 1) ) {
    encoder_stat   = CCW;
    encoder_B_lock = true;
  }
  if( encoder_B_lock ) {
    if( (temp_A == 0) && (temp_B == 0) ) {
      encoder_B_lock = false;
      result = encoder_stat;
    }
  }
  encoder_A = temp_A;
  encoder_B = temp_B;
  
  return result;
}


//------------------------------------------------------------------------------------------
// AC100のON/OFF
//------------------------------------------------------------------------------------------
void ac100_on( byte port_number )
{
  digitalWrite( port_number, HIGH );
}

void ac100_off( byte port_number )
{
  digitalWrite( port_number, LOW );
}
    
//------------------------------------------------------------------------------------------
// １桁数値を７セグメントＬＥＤ点灯パターンに変換する 
//------------------------------------------------------------------------------------------
byte num_to_segpattern( byte number )
{
  byte return_value;
  
  switch( number ) {
    case 0: return_value = Digit_0; break;
    case 1: return_value = Digit_1; break;
    case 2: return_value = Digit_2; break;
    case 3: return_value = Digit_3; break;
    case 4: return_value = Digit_4; break;
    case 5: return_value = Digit_5; break;
    case 6: return_value = Digit_6; break;
    case 7: return_value = Digit_7; break;
    case 8: return_value = Digit_8; break;
    case 9: return_value = Digit_9; break;
  }
  return return_value;
}

//------------------------------------------------------------------------------------------
// ２桁数値を２桁の７セグメント点灯パターンに変換する　小数点は点灯
//------------------------------------------------------------------------------------------
void twodig_to_segpattern( byte pressure )
{
  led_dig2 = num_to_segpattern( pressure % 10 );
  pressure /= 10;
  led_dig1 = num_to_segpattern( pressure % 10 );
  led_dig1 |= Digit_DP;
}

//------------------------------------------------------------------------------------------
// １６進数１桁数値を７セグメントＬＥＤ点灯パターンに変換する 
//------------------------------------------------------------------------------------------
byte hex_to_segpattern( byte number )
{
  byte return_value;
  
  switch( number ) {
    case  0: return_value = Digit_0; break;
    case  1: return_value = Digit_1; break;
    case  2: return_value = Digit_2; break;
    case  3: return_value = Digit_3; break;
    case  4: return_value = Digit_4; break;
    case  5: return_value = Digit_5; break;
    case  6: return_value = Digit_6; break;
    case  7: return_value = Digit_7; break;
    case  8: return_value = Digit_8; break;
    case  9: return_value = Digit_9; break;
    case 10: return_value = Digit_A; break;
    case 11: return_value = Digit_b; break;
    case 12: return_value = Digit_C; break;
    case 13: return_value = Digit_d; break;
    case 14: return_value = Digit_E; break;
    case 15: return_value = Digit_F; break;
  }
  return return_value;
}

//------------------------------------------------------------------------------------------
// １６進数２桁数値を２桁の７セグメント点灯パターンに変換する
//------------------------------------------------------------------------------------------
void twodighex_to_segpattern( byte pressure )
{
  led_dig2 = hex_to_segpattern( pressure % 16 );
  pressure /= 16;
  led_dig1 = hex_to_segpattern( pressure % 16 );
}

//------------------------------------------------------------------------------------------
// 点灯／消灯／２回点滅　５ｍｍＬＥＤ 
//------------------------------------------------------------------------------------------ 
void led_off( byte led_number )
{
  byte mask;
  
  mask = led_number;
  mask ^= 0b11111111;  // ビット反転
  led_5mm &= mask;
}

void led_on( byte led_number )
{
  led_5mm |= led_number;
}

void led_brink( byte led_number )
{
  for( int i = 1 ; i <= 2 ; i++ ) {  
    led_off( led_number );
    delay( 500 );
    led_on( led_number );
    delay( 500 );
  }
  led_off( led_number );
}

//------------------------------------------------------------------------------------------
// LEDダイナミック点灯　７セグＬＥＤ／５ｍｍ共通 
//------------------------------------------------------------------------------------------ 
void led_strobe()
{
  switch( led_pos ) {
    case 0:
      PORTD = led_dig1;
      digitalWrite( LED_DIG_1, HIGH );
      led_pos = 1;
      break;
    case 1:
      digitalWrite( LED_DIG_1, LOW  );
      PORTD = led_dig2;
      digitalWrite( LED_DIG_2, HIGH );
      led_pos = 2;
      break;
    case 2:
      digitalWrite( LED_DIG_2, LOW  );
      PORTD = led_5mm;
      digitalWrite( LED_5MM  , HIGH );
      led_pos = 3;
      break;
    case 3:
      digitalWrite( LED_5MM  , LOW );
      led_pos = 4;
      led_dimmer = DIMMER;
      break;
    case 4:
      if( 0 < led_dimmer ) {
        led_dimmer--;
      } else {
        led_pos = 0;
      }    
  }
  
  if( 0 < disp_updt_counter ) {
    disp_updt_counter--;
  }
}  

//------------------------------------------------------------------------------------------
// スイッチのON/OFF判断
//------------------------------------------------------------------------------------------ 
byte slide_sw()
{
  if( digitalRead( SLIDE_SW1 ) == HIGH ) {
    return SW_OFF;
  } else {
    return SW_ON;
  }
}

byte tact_sw( byte switch_id )
{
  if( digitalRead( switch_id ) == HIGH ) {
    return SW_OFF;
  } else {
    return SW_ON;
  }
}

//------------------------------------------------------------------------------------------
// エンコーダーによる数値入力
//  ・SW3を押して入力完了
//　・ＬＥＤ１～ＬＥＤ５を圧力に応じて点灯
// 引数：　min_val:入力最小値　max_val:入力最大値
//------------------------------------------------------------------------------------------
byte get_encoder(byte min_val, byte max_val, boolean led_display_pressure, byte init )
{
  byte turn;
  byte val;
  int pressure;
  
  val = init;
  twodig_to_segpattern( val );
  while( tact_sw(TACT_SW3) == SW_OFF ) {
    turn = rotary_encoder();
    switch( turn ) {
      case CW:
        if( val < max_val ) {
          val += 1;
          twodig_to_segpattern( val );
        }
        break;
      case CCW:
        if( min_val < val ) {
          val -= 1;
          twodig_to_segpattern( val );
        }
        break;
    }
    if( led_display_pressure ) {
      pressure = analogRead( PRESSURE_GAUGE );
      if( 10 < pressure ) { led_on( LED1 ); } else { led_off( LED1 ); }
      if( 20 < pressure ) { led_on( LED2 ); } else { led_off( LED2 ); }
      if( 30 < pressure ) { led_on( LED3 ); } else { led_off( LED3 ); }
      if( 40 < pressure ) { led_on( LED4 ); } else { led_off( LED4 ); }
      if( 50 < pressure ) { led_on( LED5 ); } else { led_off( LED5 ); }
      if( tact_sw( TACT_SW2 ) == SW_ON ) {
        ac100_on( PORT_AC2 );
      } else {
        ac100_off( PORT_AC2 );
      }
    }
  }
  return val;
}

//------------------------------------------------------------------------------------------
// モードの表示
//------------------------------------------------------------------------------------------  
void display_mode( byte mode )
{
//  led_off( LED6 );
//  led_off( LED7 );
//  led_off( LED8 );
  switch( mode ) {
    case MODE_PRESSURE_LOW:    led_off( LED6 ); led_off( LED7 ); led_on(  LED8 ); break;
    case MODE_PRESSURE_MIDIUM: led_off( LED6 ); led_on(  LED7 ); led_off( LED8 ); break;
    case MODE_PRESSURE_HIGH:   led_on(  LED6 ); led_off( LED7 ); led_off( LED8 ); break;
  }
}  

//------------------------------------------------------------------------------------------
// エンコーダーによるモード選択
//  ・モードに応じてLED6～LED8を点灯
//  ・SW3を押して入力完了
//------------------------------------------------------------------------------------------
byte get_mode( byte mode )
{
  byte turn;

  display_mode( mode );
  while( tact_sw(TACT_SW3) == SW_OFF ) {
    mode = get_mode_through( mode );
  }
  while( tact_sw(TACT_SW3) == SW_ON ) {}
  switch( mode ) {
    case MODE_PRESSURE_LOW:    led_brink( LED8 ); led_on( LED8 ); break;
    case MODE_PRESSURE_MIDIUM: led_brink( LED7 ); led_on( LED7 ); break;
    case MODE_PRESSURE_HIGH:   led_brink( LED6 ); led_on( LED6 ); break;
  }
  
  return mode;
}

//------------------------------------------------------------------------------------------
// エンコーダーによるモード選択　入力が無くてもこの処理は値を返す
//------------------------------------------------------------------------------------------
byte get_mode_through( byte mode )
{
  byte turn;

  display_mode( mode );
  turn = rotary_encoder();
  switch( turn ) {
    case CW:
      switch( mode ) {
        case MODE_PRESSURE_LOW: mode    = MODE_PRESSURE_MIDIUM; break;
        case MODE_PRESSURE_MIDIUM: mode = MODE_PRESSURE_HIGH;   break;
        case MODE_PRESSURE_HIGH: mode   = MODE_PRESSURE_LOW;    break;
      }
      display_mode( mode );
      break;
    case CCW:
      switch( mode ) {
        case MODE_PRESSURE_LOW: mode    = MODE_PRESSURE_HIGH;   break;
        case MODE_PRESSURE_MIDIUM: mode = MODE_PRESSURE_LOW;    break;
        case MODE_PRESSURE_HIGH: mode   = MODE_PRESSURE_MIDIUM; break;
      }
      display_mode( mode );
      break;
    case STOP:
      break;
  }
  
  return mode;
}

//------------------------------------------------------------------------------------------
// ON/OFF圧力をEEPROMから読み出す
//------------------------------------------------------------------------------------------  
void read_pressure_on_off( byte mode )
{
  byte address_on;
  byte address_off;

  switch( mode ) {
    case MODE_PRESSURE_LOW:
      address_on  = ADDRESS_AC_ON_L;
      address_off = ADDRESS_AC_OFF_L;
      break;
    case MODE_PRESSURE_MIDIUM:
      address_on  = ADDRESS_AC_ON_M;
      address_off = ADDRESS_AC_OFF_M;   
      break;
    case MODE_PRESSURE_HIGH:   
      address_on  = ADDRESS_AC_ON_H;
      address_off = ADDRESS_AC_OFF_H;   
      break;
  }
  pressure_ac_on      = EEPROM.read( address_on  );
  pressure_ac_off     = EEPROM.read( address_off );
}  

//------------------------------------------------------------------------------------------
// ON/OFF圧力をEEPROMへ書き込む
//------------------------------------------------------------------------------------------  
void write_pressure_on_off( byte mode, byte on_pressure, byte off_pressure )
{
  byte address_on;
  byte address_off;

  switch( mode ) {
    case MODE_PRESSURE_LOW:
      address_on  = ADDRESS_AC_ON_L;
      address_off = ADDRESS_AC_OFF_L;
      break;
    case MODE_PRESSURE_MIDIUM:
      address_on  = ADDRESS_AC_ON_M;
      address_off = ADDRESS_AC_OFF_M;   
      break;
    case MODE_PRESSURE_HIGH:   
      address_on  = ADDRESS_AC_ON_H;
      address_off = ADDRESS_AC_OFF_H;   
      break;
  }
  EEPROM.write( address_on,  on_pressure  );
  EEPROM.write( address_off, off_pressure );
}  

//------------------------------------------------------------------------------------------
// センサーの計測値(0-1024)からゲージ圧(00-99)を計算 ゲージ圧はx10した値
//------------------------------------------------------------------------------------------ 
byte calc_pressure( int ad_value )
{
  float ad_gauge;
  float gauge_pressure;
  float ad_offset;
  
  if( ad_value < 5 ) {  // センサーの出力ゼロボルト付近では圧力表示をゼロにしておく
    return 0;
  }
  ad_gauge = (float)(pressure_high_adcon - pressure_low_adcon ) / (float)(pressure_high_gauge - pressure_low_gauge);
  ad_offset = (float)pressure_low_adcon - pressure_low_gauge * ad_gauge;
  gauge_pressure = (float)(ad_value - ad_offset) / ad_gauge;

  return (byte)gauge_pressure;
}

//------------------------------------------------------------------------------------------
// 5mmLEDによる圧力表示
//------------------------------------------------------------------------------------------
void display_pressure_5mm_led( byte gauge_pressure )
{
  float pressure_step;
  float pressure_from;
  
  pressure_step = (float)(pressure_ac_off - pressure_ac_on) / 5.0;
  pressure_from = (float)pressure_ac_on;
  led_off( LED1 );
  led_off( LED2 );
  led_off( LED3 );
  led_off( LED4 );
  led_off( LED5 );
  for( int led_no = 1 ; led_no <= 5 ; led_no++ ) {
    if( pressure_from < (float)gauge_pressure ) {
      switch( led_no ) {
        case 1: led_on( LED1 ); break;
        case 2: led_on( LED2 ); break;
        case 3: led_on( LED3 ); break;
        case 4: led_on( LED4 ); break;
        case 5: led_on( LED5 ); break;
      }
    }
    pressure_from += pressure_step;
  }  
}

//------------------------------------------------------------------------------------------
// arduino setup
//------------------------------------------------------------------------------------------  
void setup()
{
  byte gauge_pressure;
 
  pinMode(PRESSURE_GAUGE, INPUT);
  pinMode(REF_VOLTAGE,    INPUT);

  pinMode(ENCODER_A, INPUT);
  pinMode(ENCODER_B, INPUT);
  encoder_A = digitalRead( ENCODER_A );
  encoder_B = digitalRead( ENCODER_B );
  
  pinMode(SLIDE_SW1, INPUT);
  pinMode(TACT_SW2,  INPUT);
  pinMode(TACT_SW3,  INPUT);
  
  digitalWrite(PORT_AC1,  LOW);
  digitalWrite(PORT_AC2,  LOW);
  pinMode(PORT_AC1,    OUTPUT);
  pinMode(PORT_AC2,    OUTPUT);
  
  pinMode(LED_SEG_A,    OUTPUT);
  pinMode(LED_SEG_B,    OUTPUT);
  pinMode(LED_SEG_C,    OUTPUT);
  pinMode(LED_SEG_D,    OUTPUT);
  pinMode(LED_SEG_E,    OUTPUT);
  pinMode(LED_SEG_F,    OUTPUT);
  pinMode(LED_SEG_G,    OUTPUT);
  pinMode(LED_SEG_DP,   OUTPUT);
  digitalWrite(LED_SEG_A,  LOW);
  digitalWrite(LED_SEG_B,  LOW);
  digitalWrite(LED_SEG_C,  LOW);
  digitalWrite(LED_SEG_D,  LOW);
  digitalWrite(LED_SEG_E,  LOW);
  digitalWrite(LED_SEG_F,  LOW);
  digitalWrite(LED_SEG_G,  LOW);
  digitalWrite(LED_SEG_DP, LOW);

  pinMode(LED_DIG_1,    OUTPUT);
  pinMode(LED_DIG_2,    OUTPUT);
  pinMode(LED_5MM,      OUTPUT);
  digitalWrite( LED_DIG_1, LOW  );
  digitalWrite( LED_DIG_2, LOW  );
  digitalWrite( LED_5MM,   LOW  );
  
  led_pos = 0;
  led_dig1 = Digit_NON;
  led_dig2 = Digit_NON;
  led_5mm  = 0x00;
  led_dimmer = DIMMER;
  disp_updt_counter = 250;
  MsTimer2::set(2, led_strobe); // 2ms毎にタイマー割り込み
  MsTimer2::start();

  if( eeprom_is_initialized() == false ) {
    EEPROM.write( ADDRESS_1ATM_AD,          DEFAULT_1ATM             );
    EEPROM.write( ADDRESS_LOW_AD_LOWBYTE,   DEFAULT_LOW_ADCON  % 256 );
    EEPROM.write( ADDRESS_LOW_AD_HIGHBYTE,  DEFAULT_LOW_ADCON  / 256 );
    EEPROM.write( ADDRESS_HIGH_AD_LOWBYTE,  DEFAULT_HIGH_ADCON % 256 );
    EEPROM.write( ADDRESS_HIGH_AD_HIGHBYTE, DEFAULT_HIGH_ADCON / 256 );
    EEPROM.write( ADDRESS_LOW_GAUGE,        DEFAULT_LOW_GAUGE        );
    EEPROM.write( ADDRESS_HIGH_GAUGE,       DEFAULT_HIGH_GAUGE       );
    EEPROM.write( ADDRESS_AC_ON_L,          DEFAULT_AC_ON_L          );
    EEPROM.write( ADDRESS_AC_OFF_L,         DEFAULT_AC_OFF_L         );    
    EEPROM.write( ADDRESS_AC_ON_M,          DEFAULT_AC_ON_M          );
    EEPROM.write( ADDRESS_AC_OFF_M,         DEFAULT_AC_OFF_M         );    
    EEPROM.write( ADDRESS_AC_ON_H,          DEFAULT_AC_ON_H          );
    EEPROM.write( ADDRESS_AC_OFF_H,         DEFAULT_AC_OFF_H         );    
    EEPROM.write( ADDRESS_MODE,             DEFAULT_MODE             );    
  }

  // EEPROMから設定値を読む
  pressure_1atm_adcon = EEPROM.read( ADDRESS_1ATM_AD );
  pressure_low_adcon  = EEPROM.read( ADDRESS_LOW_AD_HIGHBYTE );
  pressure_low_adcon  = (pressure_low_adcon << 8) + EEPROM.read( ADDRESS_LOW_AD_LOWBYTE );
  pressure_high_adcon = EEPROM.read( ADDRESS_HIGH_AD_HIGHBYTE );
  pressure_high_adcon = (pressure_high_adcon << 8) + EEPROM.read( ADDRESS_HIGH_AD_LOWBYTE );
  pressure_low_gauge  = EEPROM.read( ADDRESS_LOW_GAUGE  );
  pressure_high_gauge = EEPROM.read( ADDRESS_HIGH_GAUGE );
  pressure_mode       = EEPROM.read( ADDRESS_MODE );
  read_pressure_on_off( pressure_mode );

  // 圧力センサー校正 --------------------------------------------------
  if( (slide_sw() == SW_ON) ) {
    if( (tact_sw(TACT_SW2) == SW_ON) ) {
      // 大気圧記録 -------------------------------------
      led_on( LED8 );
      led_dig1 = Digit_MINUS;
      led_dig2 = Digit_MINUS;
      while( tact_sw(TACT_SW3) == SW_OFF ) {
        pressure_1atm_adcon = analogRead( PRESSURE_GAUGE );
        twodighex_to_segpattern( (byte)pressure_1atm_adcon );
      }
 //     pressure_1atm_adcon = analogRead( PRESSURE_GAUGE );
      led_brink( LED8 );
      while( tact_sw(TACT_SW3) == SW_ON ) {}
      // 低圧記録 ---------------------------------------
      led_on( LED7 );
      gauge_pressure = get_encoder(0,99,true,0);
      pressure_low_adcon = analogRead( PRESSURE_GAUGE );
      pressure_low_gauge = gauge_pressure;
      led_brink( LED7 );
      while( tact_sw(TACT_SW3) == SW_ON ) {}
      // 高圧記録 ---------------------------------------
      led_on( LED6 );
      gauge_pressure = get_encoder(pressure_low_gauge,99,true,0);
      pressure_high_adcon = analogRead( PRESSURE_GAUGE );
      pressure_high_gauge = gauge_pressure;
      led_brink( LED6 );
      while( tact_sw(TACT_SW3) == SW_ON ) {}
      led_off( LED1 );
      led_off( LED2 );
      led_off( LED3 );
      led_off( LED4 );
      led_off( LED5 );
      EEPROM.write( ADDRESS_1ATM_AD, pressure_1atm_adcon );
      EEPROM.write( ADDRESS_LOW_AD_LOWBYTE,   lowByte(  pressure_low_adcon  ) );
      EEPROM.write( ADDRESS_LOW_AD_HIGHBYTE,  highByte( pressure_low_adcon  ) );
      EEPROM.write( ADDRESS_HIGH_AD_LOWBYTE,  lowByte(  pressure_high_adcon ) );
      EEPROM.write( ADDRESS_HIGH_AD_HIGHBYTE, highByte( pressure_high_adcon ) ); 
      EEPROM.write( ADDRESS_LOW_GAUGE,  pressure_low_gauge  ); 
      EEPROM.write( ADDRESS_HIGH_GAUGE, pressure_high_gauge );
      eeprom_write_checksum();
    } else {
      // コンプレッサーＯＮ／ＯＦＦ圧力の設定
      if( slide_sw() == SW_ON ) {
        // モードの選択
        led_dig1 = Digit_MINUS;
        led_dig2 = Digit_MINUS;
        pressure_mode = get_mode( pressure_mode );
        read_pressure_on_off( pressure_mode );
        // コンプレッサＯＮ圧力の設定（圧力がこれ以下になるとコンプレッサをＯＮする）
        led_on( LED1 );
        pressure_ac_on = get_encoder(0,99,false,pressure_ac_on);
        led_brink( LED1 );
        while( tact_sw(TACT_SW3) == SW_ON ) {}    
        // コンプレッサＯＮ圧力の設定（圧力がこれ以下になるとコンプレッサをＯＮする）
        led_on( LED5 );
        pressure_ac_off = get_encoder(pressure_ac_on,99,false,pressure_ac_off);
        led_brink( LED5 );
        while( tact_sw(TACT_SW3) == SW_ON ) {} 
        write_pressure_on_off( pressure_mode, pressure_ac_on, pressure_ac_off );
        eeprom_write_checksum();
      }
    }
  }
}

//------------------------------------------------------------------------------------------
// arduino loop
//------------------------------------------------------------------------------------------  
void loop()                     
{
  int pressure_raw;
  byte pressure_gauge;
  byte tmp_mode;
  
  if( disp_updt_counter == 0 ) {
    pressure_raw = analogRead( PRESSURE_GAUGE );
    pressure_gauge = calc_pressure( pressure_raw );
  
    twodig_to_segpattern( pressure_gauge );
    display_pressure_5mm_led( pressure_gauge );
    if( pressure_gauge <= pressure_ac_on ) {
      ac100_on( PORT_AC2 );
    }
    if( pressure_ac_off <= pressure_gauge ) {
      ac100_off( PORT_AC2 );
    }
    disp_updt_counter = 250;  // 250 = 0.5sec
  }
  
  tmp_mode = get_mode_through( pressure_mode );
  if( tmp_mode != pressure_mode ) {
    pressure_mode = tmp_mode;
    read_pressure_on_off( pressure_mode );
  }
//  delay(500);
}

//-------- E N D   O F   P R O G R A M -----------------------------------------------------

